<?php
/**
 * Skrill Payment Overview
 *
 * The Class for Process Skrill Payment Gateways
 * Copyright (c) Skrill
 *
 * @class   SkrillPaymentOverview
 * @package Skrill/Classes
 * @located at /controllers/admin
 */
class SkrillPaymentOverview extends oxAdminList
{
    /**
     * Current Version String.
     *
     * @var string
     */
    protected $_sVersion      = 'Skrill Payment Module v1.0.10';

    /**
     * Current class template.
     *
     * @var string
     */
    protected $_sThisTemplate = "SkrillPaymentOverview.tpl";
    public $iAdminListSize = 50;
    public $blfiltering;

    protected $blDeleteOrder = false;
    protected $backOfficeError = '';
    protected $responseBackOffice = '';

    /**
     * Initialize class
     */
    public function init()
    {
        $myConfig = $this->getConfig();

        $myConfig->iAdminListSize = $this->iAdminListSize;
        parent::init();

        $this->_setCurrentListPosition(oxRegistry::getConfig()->getRequestParameter('jumppage'));
    }

    /**
     * Render interface template
     *
     * @return string template path
     */
    public function render()
    {
        $sReturn = parent::render();
        $myConfig = $this->getConfig();
        $oxShopID = $myConfig->getShopId();

        $this->oList = oxNew("oxlist");
        $this->oList->init("oxbase", "skrill__payment_logs");

        $sCountSelect = "SELECT count(*) FROM skrill__payment_logs LEFT JOIN oxorder ON skrill__payment_logs.OXORDERID = oxorder.OXID WHERE oxorder.OXSHOPID = {$oxShopID} ";
        $sWhere = $this->getWhereQuery();
        $sCountSelect .= $sWhere;

        $this->iListSize = oxDb::getDb()->getOne($sCountSelect);
        if (!$this->iCurrListPos) {
            $this->iCurrListPos = 1;
        }

        $iAdminListSize = $myConfig->getConfigParam('iAdminListSize');
        $this->_setCurrentListPosition(oxRegistry::getConfig()->getRequestParameter('jumppage'));
        $this->setLogList($sWhere);

        if ($this->iListSize > $iAdminListSize) {
            $this->setPageNavigation($iAdminListSize);
            $this->setListSize();
            $blShowNavigation = true;
        }

        // hidden default bottom custom navi
        $this->_aViewData['sHelpURL'] = false;
        $this->_aViewData['oxid'] = $oxShopID;

        if (stristr($this->getConfig()->getConfigParam('skrill__gateway_url_redirect'), 'prod')) {
            $this->_aViewData['bottom_buttons'] = 'prod';
        } else {
            $this->_aViewData['bottom_buttons'] = 'test';
        }

        $this->_aViewData['version'] = $this->_sVersion;
        $this->_aViewData['responseBackOffice'] = $this->responseBackOffice;
        $this->_aViewData['backOfficeError'] = $this->backOfficeError;

        return $sReturn;
    }

    /**
     * Set order log list
     *
     * @param string $sWhere wher query for get data from database
     */
    protected function setLogList($sWhere)
    {
        $myConfig = $this->getConfig();
        $oxShopID = $myConfig->getShopId();

        $sSelect = "SELECT skrill__payment_logs.*, concat(billlname, ', ', billfname) AS customer_name FROM skrill__payment_logs LEFT JOIN oxorder ON skrill__payment_logs.OXORDERID = oxorder.OXID WHERE oxorder.OXSHOPID = {$oxShopID} ";
        $sSelect .= $sWhere;
        $sSelect .= "order by skrill__payment_logs.id desc";

        $this->oList->iSQLRecords = $myConfig->getConfigParam('iAdminListSize');
        $this->oList->iSQLStart = $this->iCurrListPos;

        $this->oList->setSqlLimit($this->iCurrListPos, $myConfig->getConfigParam('iAdminListSize'));

        if ($this->oList->iSQLStart > $this->iListSize) {
            $this->oList->iSQLStart = 0;
        }

        $this->oList->selectString($sSelect);

        $paymentList = $this->oList->aList;

        $this->_aViewData['aLogList'] = $this->oList->aList;
        $this->_aViewData['iCount'] = $this->oList->iSQLStart;
    }

    /**
     * Set page navigation
     *
     * @param int $iAdminListSize number of shown row per page
     */
    protected function setPageNavigation($iAdminListSize)
    {
        $pagenavigation = SkrillCore::getOxidStdClass();
        $pagenavigation->pages = round(( ( $this->iListSize - 1 ) / $iAdminListSize ) + 0.5, 0);
        if (empty($pagenavigation->actpage)) {
            $pagenavigation->actpage = 0;
        }
        $pagenavigation->actpage = ($pagenavigation->actpage > $pagenavigation->pages) ? $pagenavigation->pages : round(
            ( $this->iCurrListPos / $iAdminListSize ) + 0.5,
            0
        );
        $pagenavigation->lastlink = ( $pagenavigation->pages - 1 ) * $iAdminListSize;
        $pagenavigation->nextlink = null;
        $pagenavigation->backlink = null;

        $iPos = $this->iCurrListPos + $iAdminListSize;
        if ($iPos < $this->iListSize) {
            $pagenavigation->nextlink = $iPos = $this->iCurrListPos + $iAdminListSize;
        }

        if (( $this->iCurrListPos - $iAdminListSize ) >= 0) {
            $pagenavigation->backlink = $iPos = $this->iCurrListPos - $iAdminListSize;
        }

        // calculating list start position
        $iStart = $pagenavigation->actpage - 5;
        $iStart = ( $iStart <= 0 ) ? 1 : $iStart;

        // calculating list end position
        $iEnd = $pagenavigation->actpage + 5;
        $iEnd = ( $iEnd < $iStart + 10) ? $iStart + 10 : $iEnd;
        $iEnd = ( $iEnd > $pagenavigation->pages ) ? $pagenavigation->pages : $iEnd;

        // once again adjusting start pos ..
        $iStart = ( $iEnd - 10 > 0 ) ? $iEnd - 10 : $iStart;
        $iStart = ( $pagenavigation->pages <= 11) ? 1 : $iStart;

        // navigation urls
        for ($i = $iStart; $i <= $iEnd; $i++) {
            $page = SkrillCore::getOxidStdClass();
            $page->selected = 0;
            if ($i == $pagenavigation->actpage) {
                $page->selected = 1;
            }

            $pagenavigation->changePage[$i] = $page;
        }

        $this->_aViewData['pagenavi'] = $pagenavigation;
    }

    /**
     * Set list size
     */
    protected function setListSize()
    {
        if (isset($this->iOverPos)) {
            $iPos = $this->iOverPos;
            $this->iOverPos = null;
        } else {
            $iPos = oxRegistry::getConfig()->getRequestParameter('lstrt');
        }

        if (!$iPos) {
            $iPos = 0;
        }

        $this->_aViewData['lstrt'] = $iPos;
        $this->_aViewData['listsize'] = $this->iListSize;
    }

    /**
     * Get where query
     *
     * @return string
     */
    protected function getWhereQuery()
    {
        $sWhere = "";
        if ($this->blfiltering) {
            $aFilter = $_POST['skrilllogfilter'];
            if (is_array($aFilter)) {
                foreach ($aFilter as $sKey => $sValue) {
                    if ($sValue != '') {
                        $sWhere .= ' AND ';

                        if ($sKey == 'customer_name') {
                            $sWhere .= "(";
                            $sWhere .= "skrill__payment_logs.billfname LIKE '%";
                            $sWhere .= $sValue. "%' ";
                            $sWhere .= " or ";
                            $sWhere .= "skrill__payment_logs.billlname LIKE '%";
                            $sWhere .= $sValue. "%' ";
                            $sWhere .= ")";
                        } elseif ($sKey == 'paiddate') {
                            $dateFilter = date('Y-m-d', strtotime($sValue));
                            $sWhere .= "skrill__payment_logs.".$sKey;
                            $sWhere .= " LIKE '" .$dateFilter. "%' ";
                        } else {
                            $sWhere .= "skrill__payment_logs.".$sKey;
                            $sWhere .= " LIKE '%" .$sValue. "%' ";
                        }

                        $this->_aViewData['skrilllogfilter'][$sKey] = $sValue;
                    }
                }
            }
        }
        return $sWhere;
    }

    /**
     * Set filter
     */
    public function setFilter()
    {
        $this->blfiltering = true;
    }

    /**
     * Set current list position
     *
     * @param string $sPage jump page string
     */
    protected function _setCurrentListPosition($sPage = null)
    {
        $iAdminListSize = (int) $this->getConfig()->getConfigParam('iAdminListSize');

        $iJumpTo = $sPage ? ( (int) $sPage) : (
            (int) ( (int) oxRegistry::getConfig()->getRequestParameter('lstrt') ) / $iAdminListSize
        );
        $iJumpTo = ( $sPage && $iJumpTo ) ? ( $iJumpTo - 1 ) : $iJumpTo;
        if (empty($this->iListSize)) {
            $this->iListSize = 0;
        }

        $iJumpTo = $iJumpTo * $iAdminListSize;
        if ($iJumpTo < 1) {
            $iJumpTo = 0;
        } elseif ($iJumpTo >= $this->iListSize) {
            $iJumpTo = floor($this->iListSize / $iAdminListSize - 1) * $iAdminListSize;
        }

        $this->iCurrListPos = $this->iOverPos = (int) $iJumpTo;
    }

    /**
     * Refund Payment
     */
    public function refundPayment()
    {
        SkrillLogger::set()->info('Skrill - backend refund payment');

        $oxid = oxRegistry::getConfig()->getRequestParameter('oxid');
        $paymentLog = SkrillCore::getPaymentLogById($oxid);
        $oxorderid = $paymentLog['OXORDERID'];

        $fieldParams = array();
        $fieldParams['email'] = SkrillConfig::getSetting('skrill_general_merchant_account_email');
        $fieldParams['password'] = SkrillConfig::getSetting('skrill_general_api_password');
        $fieldParams['mb_transaction_id'] = $paymentLog['mbtransactionid'];
        $fieldParams['amount'] = $paymentLog['mbamount'];
        $fieldParams['refund_status_url'] = oxRegistry::getConfig()->getSslShopUrl() .
            "index.php?cl=SkrillRefundStatus&oxid=" . $oxorderid;

        $refundPrepareResponse = SkrillCore::doRefund('prepare', $fieldParams);
        $fieldParams['password'] = substr($fieldParams['password'], 0, -5) . '*****';

        SkrillLogger::set()->info('Skrill - refund prepare parameters: ' . json_encode($fieldParams));
        SkrillLogger::set()->info('Skrill - get refund prepare response: ' . json_encode($refundPrepareResponse));

        $sid = (string) $refundPrepareResponse->sid;

        $refundResponse = SkrillCore::doRefund('refund', $sid);

        SkrillLogger::set()->info('Skrill - get refund response: ' . json_encode($refundResponse));

        $refundStatus = (string) $refundResponse->status;

        if ($refundStatus == SkrillCore::PROCESSEDSTATUS) {
            $response['status'] = SkrillCore::REFUNDEDSTATUS;
        } elseif ($refundStatus == SkrillCore::PENDINGSTATUS) {
            $response['status'] = SkrillCore::REFUNDPENDINGSTATUS;
        }

        if ($response['status'] == SkrillCore::REFUNDEDSTATUS ||
            $response['status'] == SkrillCore::REFUNDPENDINGSTATUS
        ) {
            SkrillCore::updateExistingOrderStatus($oxorderid, $response);
            SkrillCore::updatePaymentLog($oxorderid, $response);
            
            if ($response['status'] == SkrillCore::REFUNDEDSTATUS) {
                if (SkrillCore::isOxidSix()) {
                    //SkrillEmailOxidSix::sendNewOrderStatusEmail($oxorderid, $response['status']);
                } else {
                    SkrillEmail::sendNewOrderStatusEmail($oxorderid, $response['status']);
                }
            }

            $this->responseBackOffice = 'success';
            $this->backOfficeError = self::getTranslateString('SUCCESS_GENERAL_REFUND_PAYMENT');
        } else {
            $this->responseBackOffice = 'error';
            $this->backOfficeError = self::getTranslateString('ERROR_GENERAL_REFUND_PAYMENT');
        }
    }

    /**
     * Get current payment status when the status is 'pending'
     */
    public function getCurrentStatus()
    {
        SkrillLogger::set()->info('Skrill - backend update order status');

        $oxid = oxRegistry::getConfig()->getRequestParameter('oxid');
        $paymentLog = SkrillCore::getPaymentLogById($oxid);
        $transactionId = $paymentLog['transactionid'];
        $oxorderid = $paymentLog['OXORDERID'];

        $paymentResponse = SkrillCore::getPaymentResponseFromGateway($transactionId);

        SkrillLogger::set()->info('Skrill - get payment response with send request to skrill: '.
            json_encode($paymentResponse));

        if (!empty($paymentResponse['status'])) {
            SkrillCore::updateExistingOrderStatus($oxorderid, $paymentResponse);
            SkrillCore::updatePaymentLog($oxorderid, $paymentResponse);

            if ($paymentResponse['status'] == SkrillCore::PROCESSEDSTATUS) {
                SkrillEmail::sendNewOrderStatusEmail($oxorderid, $paymentResponse['status']);
            }

            $this->responseBackOffice = 'success';
            $this->backOfficeError = self::getTranslateString('SUCCESS_GENERAL_UPDATE_PAYMENT');
        } else {
            $this->responseBackOffice = 'error';
            $this->backOfficeError = self::getTranslateString('ERROR_UPDATE_BACKEND');
        }
    }

    /**
     * Delete order
     */
    public function deleteOrder()
    {
        $this->blfiltering = false;
        $this->blDeleteOrder = true;
        $oxid = oxRegistry::getConfig()->getRequestParameter('oxid');

        $sQuery = "DELETE FROM `skrill__payment_logs` WHERE OXID = ".$oxid."";
        oxDb::getDb()->Execute($sQuery);
    }

    /**
     * Translate string by code
     *
     * @param  string $langStringCode code for specific string
     * @return string
     */
    private function getTranslateString($langStringCode)
    {
        return oxRegistry::getLang()->translateString($langStringCode);
    }
}
