<?php
/**
 * Skrill Plugin Activation
 *
 * The Class for Process Skrill Payment Gateways
 * Copyright (c) Skrill
 *
 * @class   skrillactivation
 * @package Skrill/Classes
 * @located at /controllers/admin
 */
class SkrillActivation
{

    /**
     * Deactivate module
     */
    public static function onDeactivate()
    {
        self::uninstallSkrillPayments();
        self::clearTmp();
    }

    /**
     * Activate module
     */
    public static function onActivate()
    {
       // install new table and what this module need, before setup runs first time
        self::installOrderReservationTable();
        self::installPaymentLogTable();
        self::changeTableOxconfigOxvarnameLength();
        self::clearTmp();
    }

    /**
     * Uninstall plugin
     */
    public static function uninstallSkrillPayments()
    {
        $deleteQueries = array(
            // delete all oxpayments entries wich starts with skrill
            "DELETE FROM oxpayments WHERE OXID LIKE 'skrill_%'",
            // delete all oxobject2group entries wich starts with skrill
            "DELETE FROM oxobject2group WHERE OXOBJECTID LIKE 'skrill%'",
            // delete all oxobject2payment oxdelset-entries wich starts with skrill
            "DELETE FROM oxobject2payment WHERE OXPAYMENTID LIKE 'skrill%' AND OXTYPE = 'oxdelset'",
            // delete all oxobject2payment oxcountry-entries wich starts with skrill
            "DELETE FROM oxobject2payment WHERE OXPAYMENTID LIKE 'skrill%' AND OXTYPE = 'oxcountry'",
            "DELETE FROM oxconfig WHERE OXVARNAME like 'skrill_term'"
        );

        foreach ($deleteQueries as $sql) {
            oxDb::getDb()->Execute($sql);
        }
    }

    /**
     * Change/Alter field oxvarname length in table oxconfig
     */
    public static function changeTableOxconfigOxvarnameLength()
    {
        $sQuery = "ALTER TABLE `oxconfig` MODIFY `OXVARNAME` VARCHAR(100) NOT NULL DEFAULT '' COMMENT 'Variable name'";
        $oResult = oxDb::getDb()->Execute($sQuery);
    }


    /**
     * Execute Sql
     *
     * @param string $sql sql query
     */
    public static function executeSql($sql)
    {
        $result = oxDb::getDb()->Execute($sql);
        if ($result === false) {
            // query could not executed
            $this->_aViewData['skrill__sqlExecutionErrors'] .= $sql . ";\n";
        }
    }

    /**
     * get database name that already escaped as srtring type data
     * @return string
     */
    public static function getEscapeDbName()
    {
        global $ADODB_SESSION_DRIVER;
        $dbName    = oxRegistry::getConfig()->getConfigParam('dbName');
        $dbEscapeName = SkrillDb::escapeString($dbName);

        return $dbEscapeName;
    }

    /**
     * Install order reservation table
     */
    public static function installOrderReservationTable()
    {
        $tableName = 'skrill__order_number_reservations';
        $dbName    = oxRegistry::getConfig()->getConfigParam('dbName');

        $sQuery  = "SHOW TABLES FROM " . self::getEscapeDbName() ;
        $sQuery .= " LIKE '%" .$tableName. "%'";
        $oResult = SkrillDb::getInstance()->select($sQuery);
        if ($oResult->count() == 0) { // table doesn't exists
            $installSql = "CREATE TABLE `$tableName` (
                `OXID` CHAR( 32 ) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL ,
                UNIQUE (`OXID`)
                ) ENGINE = MYISAM ;";
            self::executeSql($installSql);
        }
    }

    /**
    * Install new tables, update fields and clear cache
    */
    public static function installPaymentLogTable()
    {
        $logTableName = (new SkrillPaymentLog())->_sCoreTable;

        $sQuery  = "SHOW TABLES FROM " . self::getEscapeDbName() . " LIKE '%" . $logTableName . "%'";
        $oResult = SkrillDb::getInstance()->select($sQuery);
        if ($oResult->count() == 0) { // table doesn't exists
            if (oxRegistry::getConfig()->isUtf()) {
                $sCharset = ' ENGINE=MyISAM DEFAULT CHARSET=utf8';
            } else {
                $sCharSet = ' TYPE=MyISAM';
            }

            // install sql-table
            $sQuery = "CREATE TABLE `skrill__payment_logs` (
                `OXID` varchar(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL DEFAULT '',
                `id` int NOT NULL auto_increment,
                `OXORDERID` varchar(32) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL default '',
                `orderID` int(11) NOT NULL,
                `amount` double NOT NULL default '0',
                `currency` varchar(3) NOT NULL default '',
                `mbamount` double NOT NULL default '0',
                `mbcurrency` varchar(3) NOT NULL default '',
                `ORDERDATE` datetime NOT NULL,
                `transactionid` varchar(32) NOT NULL default '',
                `mbtransactionid` varchar(32) NOT NULL default '',
                `PAIDDATE` datetime NOT NULL,
                `PAYMENTTYPE` varchar(255) NOT NULL default '',
                `PAYMENTSTATUS` varchar(50) NOT NULL default '',
                `BILLFNAME` VARCHAR(255) NOT NULL default '',
                `BILLLNAME` VARCHAR(255) NOT NULL default '',
                `paymentresponse` LONGTEXT NOT NULL default '',
                `date` timestamp NOT NULL,

                PRIMARY KEY (id)
            )". $sCharset;
            self::executeSql($sQuery);
        }

        self::addSkrillStatusColumnInOrderTable();
        self::addSkrillTransactionIdColumnInOrderTable();
    }

    /**
     * Add skrill status column in oxorder table
     */
    public static function addSkrillStatusColumnInOrderTable()
    {
        $result = SkrillDb::getInstance()->select("SHOW COLUMNS FROM `oxorder` LIKE 'skrill__status'");
        // if column doesn't exists
        if ($result->count() == 0) {
            $sql = "ALTER TABLE `oxorder` ADD `skrill__status` varchar(50) NULL";
            self::executeSql($sql);

            if (method_exists('oxDb', 'updateViews') && is_callable(array('oxDb', 'updateViews'))) {
                oxDb::getInstance()->updateViews();
            } else {
                $oDbMeta = oxNew("oxDbMetaDataHandler");
                $oDbMeta->updateViews();
            }
        }
    }

    /**
     * Clear tmp dir and smarty cache.
     *
     * @return void
     */
    public static function clearTmp()
    {
        $sTmpDir = getShopBasePath() . "/tmp/";
        $sSmartyDir = $sTmpDir . "smarty/";

        foreach (glob($sTmpDir . "*.txt") as $sFileName) {
            @unlink($sFileName);
        }
        foreach (glob($sSmartyDir . "*.php") as $sFileName) {
            @unlink($sFileName);
        }
    }

    /**
     * Add skrill transaction id column in oxorder table
     */
    public static function addSkrillTransactionIdColumnInOrderTable()
    {
        $result = SkrillDb::getInstance()->select("SHOW COLUMNS FROM `oxorder` LIKE 'skrill__transaction_id'");
        // if column doesn't exists
        if ($result->count() == 0) {
            $sql = "ALTER TABLE `oxorder` ADD `skrill__transaction_id` varchar(32) NULL";
            self::executeSql($sql);

            if (method_exists('oxDb', 'updateViews') && is_callable(array('oxDb', 'updateViews'))) {
                oxDb::getInstance()->updateViews();
            } else {
                $oDbMeta = oxNew("oxDbMetaDataHandler");
                $oDbMeta->updateViews();
            }
        }
    }
}
